using System.Collections.Generic;
using System.ComponentModel;

namespace Dapfor.Net.Grid.Tutorial.BusinessData.Currencies
{
    [TypeDescriptionProvider(typeof(CurrencyRateTypeDescriptorProvider))]
    public class CurrencyRate : INotifyPropertyChanged
    {
        #region Fields

        private readonly IList<string> _availableCurrencies;
        private readonly string _mainCurrency;
        private readonly Dictionary<string, double> _prevRates = new Dictionary<string, double>();
        private readonly Dictionary<string, double> _rates = new Dictionary<string, double>();
        private readonly object _synchro = new object();

        #endregion

        #region Constructor

        public CurrencyRate(string mainCurrency, IList<string> availableCurrencies)
        {
            _availableCurrencies = availableCurrencies;

            //Default rate is 1.
            _mainCurrency = mainCurrency;
            foreach (string currency in availableCurrencies)
            {
                _rates.Add(currency, 1);
            }
            foreach (string currency in availableCurrencies)
            {
                _prevRates.Add(currency, 1);
            }
        }

        #endregion

        #region Public properties

        [Browsable(false)]
        public IList<string> AvailableCurrencies
        {
            get { return _availableCurrencies; }
        }

        public CurrencyPairRate this[string currency]
        {
            get { return new CurrencyPairRate(currency, this); }
        }

        [Browsable(false)]
        public string MainCurrency
        {
            get { return _mainCurrency; }
        }

        #endregion

        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;

        #endregion

        #region Nested type: CurrencyPairRate

        public class CurrencyPairRate
        {
            private readonly string _currency2;
            private readonly CurrencyRate _owner;


            public CurrencyPairRate(string currency2, CurrencyRate owner)
            {
                _currency2 = currency2;
                _owner = owner;
            }

            public string Currency1
            {
                get { return _owner._mainCurrency; }
            }

            public string Currency2
            {
                get { return _currency2; }
            }

            public double PrevRate
            {
                get { return _owner._prevRates[_currency2]; }
            }

            public double Rate
            {
                get
                {
                    //Protect _rates & _prevRates containers against race conditions
                    lock (_owner._synchro)
                    {
                        return _owner._rates[_currency2];
                    }
                }
                set
                {
                    //Protect _rates & _prevRates containers against race conditions
                    lock (_owner._synchro)
                    {
                        _owner._prevRates[_currency2] = _owner._rates[_currency2];
                        _owner._rates[_currency2] = value;
                    }

                    if (_owner.PropertyChanged != null)
                    {
                        _owner.PropertyChanged(_owner, new PropertyChangedEventArgs(_currency2));
                    }
                }
            }
        }

        #endregion
    }
}